# License: Apache-2.0
import databricks.koalas as ks
import numpy as np
import pandas as pd
import pytest
from pandas.testing import assert_frame_equal

from gators.feature_selection.supervized_correlation_filter import (
    SupervizedCorrelationFilter,
)
ks.set_option("compute.default_index_type", "distributed-sequence")


@pytest.fixture
def data_ks():
    max_corr = 0.8
    X = ks.DataFrame(
        {
            "A": [7.25, 71.2833, 7.925, 53.1, 8.05, 8.4583],
            "B": [1, 1, 0, 1, 0, 0],
            "D": [22.0, 38.0, 26.0, 35.0, 35.0, 31.2],
            "F": [3, 1, 2, 1, 2, 3],
        }
    )
    X_expected = X[["A", "B", "D"]].to_pandas()
    feature_importances = ks.Series({"A": 1, "B": 0.8, "D": 0.7, "F": 0.1})
    obj = SupervizedCorrelationFilter(
        feature_importances=feature_importances, max_corr=max_corr
    ).fit(X)
    return obj, X, X_expected


@pytest.mark.koalas
def test_ks(data_ks):
    obj, X, X_expected = data_ks
    X_new = obj.transform(X)
    assert_frame_equal(X_new.to_pandas(), X_expected)


@pytest.mark.koalas
def test_ks_np(data_ks):
    obj, X, X_expected = data_ks
    X_numpy_new = obj.transform_numpy(X.to_numpy())
    X_new = pd.DataFrame(X_numpy_new, columns=X_expected.columns)
    assert_frame_equal(X_new, X_expected.astype(np.float64))
